@@ -28,11 +28,8 @@
 #include <linux/mount.h>
 #include <linux/slab.h>
 #include <linux/init.h>
-#include <linux/version.h>
 #include <linux/list.h>
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 #include <linux/seq_file.h>
-#endif
 #include <linux/vfs.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
@@ -65,7 +62,7 @@ unsigned int CIFSMaximumBufferSize = CIFS_MAX_MSGSIZE;
 struct task_struct * oplockThread = NULL;
 
 extern int cifs_mount(struct super_block *, struct cifs_sb_info *, char *,
-			char *);
+			const char *);
 extern int cifs_umount(struct super_block *, struct cifs_sb_info *);
 void cifs_proc_init(void);
 void cifs_proc_clean(void);
@@ -73,18 +70,23 @@ void cifs_proc_clean(void);
 static DECLARE_COMPLETION(cifs_oplock_exited);
 
 
-struct super_block *
-cifs_read_super(struct super_block *sb, void *data, int silent)
+static int
+cifs_read_super(struct super_block *sb, void *data,
+		const char *devname, int silent)
 {
 	struct inode *inode;
 	struct cifs_sb_info *cifs_sb;
 	int rc = 0;
 
+	sb->s_fs_info = kmalloc(sizeof(struct cifs_sb_info),GFP_KERNEL);
 	cifs_sb = CIFS_SB(sb);
 	if(cifs_sb == NULL)
-		return 0;
+		return -ENOMEM;
+	else
+		memset(cifs_sb,0,sizeof(struct cifs_sb_info));
+	
 
-	rc = cifs_mount(sb, cifs_sb, data, NULL);
+	rc = cifs_mount(sb, cifs_sb, data, devname);
 
 	if (rc) {
 		if (!silent)
@@ -104,15 +106,19 @@ cifs_read_super(struct super_block *sb, void *data, int silent)
 	sb->s_blocksize_bits = 14;	/* default 2**14 = CIFS_MAX_MSGSIZE */
 	inode = iget(sb, ROOT_I);
 
-	if (!inode)
+	if (!inode) {
+		rc = -ENOMEM;
 		goto out_no_root;
+	}
 
 	sb->s_root = d_alloc_root(inode);
 
-	if (!sb->s_root)
+	if (!sb->s_root) {
+		rc = -ENOMEM;
 		goto out_no_root;
+	}
 
-	return sb;
+	return 0;
 
 out_no_root:
 	cERROR(1, ("cifs_read_super: get root inode failed"));
@@ -121,12 +127,13 @@ out_no_root:
 
 out_mount_failed:
 	if(cifs_sb->local_nls)
-		unload_nls(cifs_sb->local_nls);
-	sb->s_dev = 0;
-	return 0;
+		unload_nls(cifs_sb->local_nls);	
+	if(cifs_sb)
+		kfree(cifs_sb);
+	return rc;
 }
 
-void
+static void
 cifs_put_super(struct super_block *sb)
 {
 	int rc = 0;
@@ -143,11 +150,12 @@ cifs_put_super(struct super_block *sb)
 		cERROR(1, ("cifs_umount failed with return code %d", rc));
 	}
 	unload_nls(cifs_sb->local_nls);
+	kfree(cifs_sb);
 	return;
 }
 
-int
-cifs_statfs(struct super_block *sb, struct statfs *buf)
+static int
+cifs_statfs(struct super_block *sb, struct kstatfs *buf)
 {
 	int xid, rc;
 	struct cifs_sb_info *cifs_sb;
@@ -179,36 +187,39 @@ cifs_statfs(struct super_block *sb, struct statfs *buf)
 	return 0;		/* always return success? what if volume is no longer available? */
 }
 
-static int cifs_permission(struct inode * inode, int mask)
+static int cifs_permission(struct inode * inode, int mask, struct nameidata *nd)
 {
 	/* the server does permission checks, we do not need to do it here */
 	return 0;
 }
 
+static kmem_cache_t *cifs_inode_cachep;
 kmem_cache_t *cifs_req_cachep;
 kmem_cache_t *cifs_mid_cachep;
 kmem_cache_t *cifs_oplock_cachep;
 
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
 static struct inode *
 cifs_alloc_inode(struct super_block *sb)
 {
-        struct cifsInodeInfo *cifs_inode;
-        cifs_inode =
-            (struct cifsInodeInfo *) kmem_cache_alloc(cifs_inode_cachep,
-                                                      SLAB_KERNEL);
-        if (!cifs_inode)
-                return NULL;
-        cifs_inode->cifsAttrs = 0x20;   /* default */
-        atomic_set(&cifs_inode->inUse, 0);
-        cifs_inode->time = 0;
-        /* Until the file is open and we have gotten oplock
-        info back from the server, can not assume caching of
-        file data or metadata */
-        cifs_inode->clientCanCacheRead = FALSE;
-        cifs_inode->clientCanCacheAll = FALSE;
-        INIT_LIST_HEAD(&cifs_inode->openFileList);
-        return &cifs_inode->vfs_inode;
+	struct cifsInodeInfo *cifs_inode;
+	cifs_inode =
+	    (struct cifsInodeInfo *) kmem_cache_alloc(cifs_inode_cachep,
+						      SLAB_KERNEL);
+	if (!cifs_inode)
+		return NULL;
+	cifs_inode->cifsAttrs = 0x20;	/* default */
+	atomic_set(&cifs_inode->inUse, 0);
+	cifs_inode->time = 0;
+	/* Until the file is open and we have gotten oplock
+	info back from the server, can not assume caching of
+	file data or metadata */
+	cifs_inode->clientCanCacheRead = FALSE;
+	cifs_inode->clientCanCacheAll = FALSE;
+	cifs_inode->vfs_inode.i_blksize = CIFS_MAX_MSGSIZE;
+	cifs_inode->vfs_inode.i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
+
+	INIT_LIST_HEAD(&cifs_inode->openFileList);
+	return &cifs_inode->vfs_inode;
 }
 
 static void
@@ -216,14 +227,12 @@ cifs_destroy_inode(struct inode *inode)
 {
 	kmem_cache_free(cifs_inode_cachep, CIFS_I(inode));
 }
-#endif
 
 /*
  * cifs_show_options() is for displaying mount options in /proc/mounts.
  * Not all settable options are displayed but most of the important
  * ones are.
  */
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 static int
 cifs_show_options(struct seq_file *s, struct vfsmount *m)
 {
@@ -246,133 +255,128 @@ cifs_show_options(struct seq_file *s, struct vfsmount *m)
 	}
 	return 0;
 }
-#endif
 
 #ifdef CIFS_QUOTA
 int cifs_xquota_set(struct super_block * sb, int quota_type, qid_t qid,
-                struct fs_disk_quota * pdquota)
+		struct fs_disk_quota * pdquota)
 {
-        int xid;
-        int rc = 0;
-        struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
-        struct cifsTconInfo *pTcon;
-
-        if(cifs_sb)
-                pTcon = cifs_sb->tcon;
-        else
-                return -EIO;
+	int xid;
+	int rc = 0;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
+	struct cifsTconInfo *pTcon;
+	
+	if(cifs_sb)
+		pTcon = cifs_sb->tcon;
+	else
+		return -EIO;
 
 
-        xid = GetXid();
-        if(pTcon) {
-                cFYI(1,("set type: 0x%x id: %d",quota_type,qid));
-        } else {
-                return -EIO;
-        }
+	xid = GetXid();
+	if(pTcon) {
+		cFYI(1,("set type: 0x%x id: %d",quota_type,qid));		
+	} else {
+		return -EIO;
+	}
 
-        FreeXid(xid);
-        return rc;
+	FreeXid(xid);
+	return rc;
 }
 
 int cifs_xquota_get(struct super_block * sb, int quota_type, qid_t qid,
                 struct fs_disk_quota * pdquota)
 {
-        int xid;
-        int rc = 0;
-        struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
-        struct cifsTconInfo *pTcon;
+	int xid;
+	int rc = 0;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
+	struct cifsTconInfo *pTcon;
 
-        if(cifs_sb)
-                pTcon = cifs_sb->tcon;
-        else
-                return -EIO;
+	if(cifs_sb)
+		pTcon = cifs_sb->tcon;
+	else
+		return -EIO;
 
-        xid = GetXid();
-        if(pTcon) {
+	xid = GetXid();
+	if(pTcon) {
                 cFYI(1,("set type: 0x%x id: %d",quota_type,qid));
-        } else {
-                rc = -EIO;
-        }
+	} else {
+		rc = -EIO;
+	}
 
-        FreeXid(xid);
-        return rc;
+	FreeXid(xid);
+	return rc;
 }
 
 int cifs_xstate_set(struct super_block * sb, unsigned int flags, int operation)
 {
-        int xid;
-        int rc = 0;
-        struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
-        struct cifsTconInfo *pTcon;
+	int xid; 
+	int rc = 0;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
+	struct cifsTconInfo *pTcon;
 
-        if(cifs_sb)
-                pTcon = cifs_sb->tcon;
-        else
-                return -EIO;
+	if(cifs_sb)
+		pTcon = cifs_sb->tcon;
+	else
+		return -EIO;
 
-        xid = GetXid();
-        if(pTcon) {
+	xid = GetXid();
+	if(pTcon) {
                 cFYI(1,("flags: 0x%x operation: 0x%x",flags,operation));
-        } else {
-                rc = -EIO;
-        }
+	} else {
+		rc = -EIO;
+	}
 
-        FreeXid(xid);
-        return rc;
+	FreeXid(xid);
+	return rc;
 }
 
 int cifs_xstate_get(struct super_block * sb, struct fs_quota_stat *qstats)
 {
-        int xid;
-        int rc = 0;
-        struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
-        struct cifsTconInfo *pTcon;
+	int xid;
+	int rc = 0;
+	struct cifs_sb_info *cifs_sb = CIFS_SB(sb);
+	struct cifsTconInfo *pTcon;
 
-        if(cifs_sb) {
-                pTcon = cifs_sb->tcon;
-        } else {
-                return -EIO;
-        }
-        xid = GetXid();
-        if(pTcon) {
-                cFYI(1,("pqstats %p",qstats));
-        } else {
-                rc = -EIO;
-        }
+	if(cifs_sb) {
+		pTcon = cifs_sb->tcon;
+	} else {
+		return -EIO;
+	}
+	xid = GetXid();
+	if(pTcon) {
+		cFYI(1,("pqstats %p",qstats));		
+	} else {
+		rc = -EIO;
+	}
 
-        FreeXid(xid);
-        return rc;
+	FreeXid(xid);
+	return rc;
 }
 
 static struct quotactl_ops cifs_quotactl_ops = {
-        .set_xquota     = cifs_xquota_set,
-        .get_xquota     = cifs_xquota_set,
-        .set_xstate     = cifs_xstate_set,
-        .get_xstate     = cifs_xstate_get,
+	.set_xquota	= cifs_xquota_set,
+	.get_xquota	= cifs_xquota_set,
+	.set_xstate	= cifs_xstate_set,
+	.get_xstate	= cifs_xstate_get,
 };
 #endif
 
-
 struct super_operations cifs_super_ops = {
 	.read_inode = cifs_read_inode,
 	.put_super = cifs_put_super,
 	.statfs = cifs_statfs,
-/*	.alloc_inode = cifs_alloc_inode,
-	.destroy_inode = cifs_destroy_inode, */
+	.alloc_inode = cifs_alloc_inode,
+	.destroy_inode = cifs_destroy_inode,
 /*	.drop_inode	    = generic_delete_inode, 
 	.delete_inode	= cifs_delete_inode,  *//* Do not need the above two functions     
    unless later we add lazy close of inodes or unless the kernel forgets to call
    us with the same number of releases (closes) as opens */
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 	.show_options = cifs_show_options,
-#endif
 /*    .umount_begin   = cifs_umount_begin, *//* consider adding in the future */
 };
 
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
 static struct super_block *
 cifs_get_sb(struct file_system_type *fs_type,
-	    int flags, char *dev_name, void *data)
+	    int flags, const char *dev_name, void *data)
 {
 	int rc;
 	struct super_block *sb = sget(fs_type, NULL, set_anon_super, NULL);
@@ -394,24 +398,54 @@ cifs_get_sb(struct file_system_type *fs_type,
 	return sb;
 }
 
-ssize_t
+static ssize_t
 cifs_read_wrapper(struct file * file, char *read_data, size_t read_size,
           loff_t * poffset)
 {
-	if(CIFS_I(file->f_dentry->d_inode)->clientCanCacheRead)
+	if(file == NULL)
+		return -EIO;
+	else if(file->f_dentry == NULL)
+		return -EIO;
+	else if(file->f_dentry->d_inode == NULL)
+		return -EIO;
+
+	if(CIFS_I(file->f_dentry->d_inode)->clientCanCacheRead) {
 		return generic_file_read(file,read_data,read_size,poffset);
-	else
-		return cifs_read(file,read_data,read_size,poffset);	
+	} else {
+		/* BB do we need to lock inode from here until after invalidate? */
+		if(file->f_dentry->d_inode->i_mapping) {
+			filemap_fdatawrite(file->f_dentry->d_inode->i_mapping);
+			filemap_fdatawait(file->f_dentry->d_inode->i_mapping);
+		}
+		/* force read from the server since we do not have oplock */
+		/* BB we should relax this and do it on a timer and call cifs_revalidate */
+		/* BB we should make timer configurable */
+		invalidate_remote_inode(file->f_dentry->d_inode);
+		return generic_file_read(file,read_data,read_size,poffset);
+	}
 }
 
-ssize_t
+static ssize_t
 cifs_write_wrapper(struct file * file, const char *write_data,
            size_t write_size, loff_t * poffset) 
 {
-	if(CIFS_I(file->f_dentry->d_inode)->clientCanCacheAll)    /* check caching for write */
-		return generic_file_write(file,write_data, write_size,poffset);
-	else
-		return cifs_write(file,write_data,write_size,poffset);
+	ssize_t written;
+
+	if(file == NULL)
+		return -EIO;
+	else if(file->f_dentry == NULL)
+		return -EIO;
+	else if(file->f_dentry->d_inode == NULL)
+		return -EIO;
+
+	/* check whether we can cache writes locally */
+	written = generic_file_write(file,write_data,write_size,poffset);
+	if(!CIFS_I(file->f_dentry->d_inode)->clientCanCacheAll)  {
+		if(file->f_dentry->d_inode->i_mapping) {
+			filemap_fdatawrite(file->f_dentry->d_inode->i_mapping);
+		}
+	}
+	return written;
 }
 
 
@@ -422,30 +456,26 @@ static struct file_system_type cifs_fs_type = {
 	.kill_sb = kill_anon_super,
 	/*  .fs_flags */
 };
-#endif
-
-static DECLARE_FSTYPE(cifs_fs_type, "cifs", cifs_read_super,0);
-
-
 struct inode_operations cifs_dir_inode_ops = {
 	.create = cifs_create,
 	.lookup = cifs_lookup,
+	.getattr = cifs_getattr,
 	.unlink = cifs_unlink,
 	.link = cifs_hardlink,
 	.mkdir = cifs_mkdir,
 	.rmdir = cifs_rmdir,
 	.rename = cifs_rename,
 	.permission = cifs_permission,
-	.revalidate = cifs_revalidate,  
+/*	revalidate:cifs_revalidate,   */
 	.setattr = cifs_setattr,
 	.symlink = cifs_symlink,
 	.mknod   = cifs_mknod,
 };
 
 struct inode_operations cifs_file_inode_ops = {
-	.revalidate = cifs_revalidate,
+/*	revalidate:cifs_revalidate, */
 	.setattr = cifs_setattr,
-/*	.getattr = cifs_getattr,*/
+	.getattr = cifs_getattr, /* do we need this anymore? */
 	.rename = cifs_rename,
 	.permission = cifs_permission,
 #ifdef CIFS_XATTR
@@ -472,15 +502,15 @@ struct inode_operations cifs_symlink_inode_ops = {
 };
 
 struct file_operations cifs_file_ops = {
-	.read = generic_file_read,
-	.write = generic_file_write, 
+	.read = cifs_read_wrapper,
+	.write = cifs_write_wrapper, 
 	.open = cifs_open,
 	.release = cifs_close,
 	.lock = cifs_lock,
 	.fsync = cifs_fsync,
 	.flush = cifs_flush,
 	.mmap  = cifs_file_mmap,
-/*	.sendfile = generic_file_sendfile,*/
+	.sendfile = generic_file_sendfile,
 };
 
 struct file_operations cifs_dir_ops = {
@@ -488,7 +518,7 @@ struct file_operations cifs_dir_ops = {
 	.release = cifs_closedir,
 	.read    = generic_read_dir,
 };
-/*
+
 static void
 cifs_init_once(void *inode, kmem_cache_t * cachep, unsigned long flags)
 {
@@ -501,30 +531,27 @@ cifs_init_once(void *inode, kmem_cache_t * cachep, unsigned long flags)
 	}
 }
 
-int
+static int
 cifs_init_inodecache(void)
 {
 	cifs_inode_cachep = kmem_cache_create("cifs_inode_cache",
 					      sizeof (struct cifsInodeInfo),
-					      0, SLAB_HWCACHE_ALIGN,
+					      0, SLAB_HWCACHE_ALIGN|SLAB_RECLAIM_ACCOUNT,
 					      cifs_init_once, NULL);
 	if (cifs_inode_cachep == NULL)
 		return -ENOMEM;
-	else
-		memset(cifs_sb,0,sizeof(struct cifs_sb_info));
-	
 
 	return 0;
 }
 
-void
+static void
 cifs_destroy_inodecache(void)
 {
 	if (kmem_cache_destroy(cifs_inode_cachep))
 		printk(KERN_WARNING "cifs_inode_cache: error freeing\n");
-} */
+}
 
-int
+static int
 cifs_init_request_bufs(void)
 {
 	cifs_req_cachep = kmem_cache_create("cifs_request",
@@ -537,7 +564,7 @@ cifs_init_request_bufs(void)
 	return 0;
 }
 
-void
+static void
 cifs_destroy_request_bufs(void)
 {
 	if (kmem_cache_destroy(cifs_req_cachep))
@@ -545,7 +572,7 @@ cifs_destroy_request_bufs(void)
 		       "cifs_destroy_request_cache: error not all structures were freed\n");
 }
 
-int
+static int
 cifs_init_mids(void)
 {
 	cifs_mid_cachep = kmem_cache_create("cifs_mpx_ids",
@@ -553,7 +580,7 @@ cifs_init_mids(void)
 				SLAB_HWCACHE_ALIGN, NULL, NULL);
 	if (cifs_mid_cachep == NULL)
 		return -ENOMEM;
-	cifs_oplock_cachep = kmem_cache_create("cifs_oplock_struct",
+	cifs_oplock_cachep = kmem_cache_create("cifs_oplock_structs",
 				sizeof (struct oplock_q_entry), 0,
 				SLAB_HWCACHE_ALIGN, NULL, NULL);
 	if (cifs_oplock_cachep == NULL) {
@@ -564,7 +591,7 @@ cifs_init_mids(void)
 	return 0;
 }
 
-void
+static void
 cifs_destroy_mids(void)
 {
 	if (kmem_cache_destroy(cifs_mid_cachep))
@@ -581,23 +608,23 @@ static int cifs_oplock_thread(void * dummyarg)
 	struct cifsTconInfo *pTcon;
 	struct inode * inode;
 	__u16  netfid;
-	int rc = 0;
+	int rc;
+
+	daemonize("cifsoplockd");
+	allow_signal(SIGTERM);
 
-	daemonize();
-	sprintf(current->comm,"cifsoplockd");
 	oplockThread = current;
 	do {
 		set_current_state(TASK_INTERRUPTIBLE);
-
-		schedule_timeout(3*HZ);
+		
+		schedule_timeout(1*HZ);  
 		spin_lock(&GlobalMid_Lock);
 		if(list_empty(&GlobalOplock_Q)) {
 			spin_unlock(&GlobalMid_Lock);
-			schedule_timeout(7*HZ);
+			schedule_timeout(39*HZ);
 		} else {
-			oplock_item = list_entry(GlobalOplock_Q.next,
+			oplock_item = list_entry(GlobalOplock_Q.next, 
 				struct oplock_q_entry, qhead);
-
 			if(oplock_item) {
 				pTcon = oplock_item->tcon;
 				inode = oplock_item->pinode;
@@ -605,23 +632,28 @@ static int cifs_oplock_thread(void * dummyarg)
 				spin_unlock(&GlobalMid_Lock);
 				DeleteOplockQEntry(oplock_item);
 				if (S_ISREG(inode->i_mode)) {
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
-					rc = filemap_fdatasync(inode->i_mapping);
-					if(rc)
-						CIFS_I(inode)->write_behind_rc = rc;
-#else 
-					filemap_fdatasync(inode->i_mapping);
-#endif
-					if(CIFS_I(inode)->clientCanCacheRead == 0) 
-						invalidate_inode_pages(inode);
+					rc = filemap_fdatawrite(inode->i_mapping);
+					if(CIFS_I(inode)->clientCanCacheRead == 0)
+						invalidate_remote_inode(inode);
 				} else
 					rc = 0;
-				rc = CIFSSMBLock(0, pTcon, 
-					netfid,
-					0 /* len */ , 0 /* offset */, 0, 
-					0, LOCKING_ANDX_OPLOCK_RELEASE,
-					0 /* wait flag */);
-				cFYI(1,("Oplock release rc = %d ",rc));
+				if (rc)
+					CIFS_I(inode)->write_behind_rc = rc;
+				cFYI(1,("Oplock flush inode %p rc %d",inode,rc));
+
+				/* releasing a stale oplock after recent reconnection 
+				of smb session using a now incorrect file 
+				handle is not a data integrity issue but do  
+				not bother sending an oplock release if session 
+				to server still is disconnected since oplock 
+				already released by the server in that case */
+				if(pTcon->tidStatus != CifsNeedReconnect) {
+				    rc = CIFSSMBLock(0, pTcon, netfid,
+					    0 /* len */ , 0 /* offset */, 0, 
+					    0, LOCKING_ANDX_OPLOCK_RELEASE,
+					    0 /* wait flag */);
+					cFYI(1,("Oplock release rc = %d ",rc));
+				}
 			} else
 				spin_unlock(&GlobalMid_Lock);
 		}
@@ -633,7 +665,7 @@ static int __init
 init_cifs(void)
 {
 	int rc = 0;
-#if CONFIG_PROC_FS
+#ifdef CONFIG_PROC_FS
 	cifs_proc_init();
 #endif
 	INIT_LIST_HEAD(&GlobalServerList);	/* BB not implemented yet */
@@ -656,7 +688,7 @@ init_cifs(void)
 	GlobalSMBSeslock = RW_LOCK_UNLOCKED;
 	GlobalMid_Lock = SPIN_LOCK_UNLOCKED;
 
-/*	rc = cifs_init_inodecache();*/
+	rc = cifs_init_inodecache();
 	if (!rc) {
 		rc = cifs_init_mids();
 		if (!rc) {
@@ -672,9 +704,9 @@ init_cifs(void)
 			}
 			cifs_destroy_mids();
 		}
-/*		cifs_destroy_inodecache(); */
+		cifs_destroy_inodecache();
 	}
-#if CONFIG_PROC_FS
+#ifdef CONFIG_PROC_FS
 	cifs_proc_clean();
 #endif
 	return rc;
@@ -684,11 +716,11 @@ static void __exit
 exit_cifs(void)
 {
 	cFYI(0, ("In unregister ie exit_cifs"));
-#if CONFIG_PROC_FS
+#ifdef CONFIG_PROC_FS
 	cifs_proc_clean();
 #endif
-    	unregister_filesystem(&cifs_fs_type);
-/*	cifs_destroy_inodecache();*/
+	unregister_filesystem(&cifs_fs_type);
+	cifs_destroy_inodecache();
 	cifs_destroy_mids();
 	cifs_destroy_request_bufs();
 	if(oplockThread) {
