@@ -23,12 +23,10 @@
 #include <linux/fs.h>
 #include <linux/stat.h>
 #include <linux/fcntl.h>
-#include <linux/version.h>
 #include <linux/pagemap.h>
+#include <linux/pagevec.h>
 #include <linux/smp_lock.h>
-#include <linux/list.h>
 #include <asm/div64.h>
-#include <linux/mm.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
 #include "cifsglob.h"
@@ -46,6 +44,7 @@ cifs_open(struct inode *inode, struct file *file)
 	struct cifsTconInfo *pTcon;
 	struct cifsFileInfo *pCifsFile;
 	struct cifsInodeInfo *pCifsInode;
+	struct list_head * tmp;
 	char *full_path = NULL;
 	int desiredAccess = 0x20197;
 	int disposition;
@@ -54,12 +53,36 @@ cifs_open(struct inode *inode, struct file *file)
 
 	xid = GetXid();
 
-	cFYI(1, (" inode = 0x%p file flags are %x", inode, file->f_flags));
 	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 
+	if (file->f_flags & O_CREAT) {
+		/* search inode for this file and fill in file->private_data = */
+		pCifsInode = CIFS_I(file->f_dentry->d_inode);
+		read_lock(&GlobalSMBSeslock);
+		list_for_each(tmp, &pCifsInode->openFileList) {            
+			pCifsFile = list_entry(tmp,struct cifsFileInfo, flist);           
+			if((pCifsFile->pfile == NULL)&& (pCifsFile->pid = current->pid)){		
+			/* set mode ?? */
+				pCifsFile->pfile = file; /* needed for writepage */
+				file->private_data = pCifsFile;
+				break;
+			}
+		}
+		read_unlock(&GlobalSMBSeslock);
+		if(file->private_data != NULL) {
+			rc = 0;
+			FreeXid(xid);
+			return rc;
+		} else {
+			if(file->f_flags & O_EXCL)
+				cERROR(1,("could not find file instance for new file %p ",file));
+		}
+	}
+
 	full_path = build_path_from_dentry(file->f_dentry);
 
+	cFYI(1, (" inode = 0x%p file flags are 0x%x for %s", inode, file->f_flags,full_path));
 	if ((file->f_flags & O_ACCMODE) == O_RDONLY)
 		desiredAccess = GENERIC_READ;
 	else if ((file->f_flags & O_ACCMODE) == O_WRONLY)
@@ -121,7 +144,6 @@ cifs_open(struct inode *inode, struct file *file)
 		FreeXid(xid);
 		return -ENOMEM;
 	}
-
 	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
 			CREATE_NOT_DIR, &netfid, &oplock, buf, cifs_sb->local_nls);
 	if (rc) {
@@ -141,14 +163,14 @@ cifs_open(struct inode *inode, struct file *file)
 			pCifsFile->pInode = inode;
 			pCifsFile->invalidHandle = FALSE;
 			pCifsFile->closePend     = FALSE;
+			write_lock(&file->f_owner.lock);
 			write_lock(&GlobalSMBSeslock);
-			spin_lock(&files_lock);
 			list_add(&pCifsFile->tlist,&pTcon->openFileList);
 			pCifsInode = CIFS_I(file->f_dentry->d_inode);
 			if(pCifsInode) {
 				list_add(&pCifsFile->flist,&pCifsInode->openFileList);
-				write_unlock(&files_lock);
 				write_unlock(&GlobalSMBSeslock);
+				write_unlock(&file->f_owner.lock);
 
 		                if (pTcon->ses->capabilities & CAP_UNIX)
 					rc = cifs_get_inode_info_unix(&file->f_dentry->d_inode,
@@ -164,10 +186,9 @@ cifs_open(struct inode *inode, struct file *file)
 				} else if(oplock == OPLOCK_READ)
 					pCifsInode->clientCanCacheRead = TRUE;
 			} else {
-				write_unlock(&files_lock);
 				write_unlock(&GlobalSMBSeslock);
+				write_unlock(&file->f_owner.lock);
 			}
-
 			if(file->f_flags & O_CREAT) {           
 				/* time to set mode which we can not set earlier due
 				 to problems creating new read-only files */
@@ -207,105 +228,109 @@ static int cifs_relock_file(struct cifsFileInfo * cifsFile)
 
 static int cifs_reopen_file(struct inode *inode, struct file *file)
 {
-        int rc = -EACCES;
-        int xid, oplock;
-        struct cifs_sb_info *cifs_sb;
-        struct cifsTconInfo *pTcon;
-        struct cifsFileInfo *pCifsFile;
-        struct cifsInodeInfo *pCifsInode;
-        char *full_path = NULL;
-        int desiredAccess = 0x20197;
-        int disposition = FILE_OPEN;
-        __u16 netfid;
-        FILE_ALL_INFO * buf = NULL;
+	int rc = -EACCES;
+	int xid, oplock;
+	struct cifs_sb_info *cifs_sb;
+	struct cifsTconInfo *pTcon;
+	struct cifsFileInfo *pCifsFile;
+	struct cifsInodeInfo *pCifsInode;
+	char *full_path = NULL;
+	int desiredAccess = 0x20197;
+	int disposition = FILE_OPEN;
+	__u16 netfid;
+	FILE_ALL_INFO * buf = NULL;
 
-        if(inode == NULL)
-                return -EBADF;
-        if (file->private_data) {
-                pCifsFile = (struct cifsFileInfo *) file->private_data;
-        } else
-                return -EBADF;
+	if(inode == NULL)
+		return -EBADF;
+	if (file->private_data) {
+		pCifsFile = (struct cifsFileInfo *) file->private_data;
+	} else
+		return -EBADF;
 
-        xid = GetXid();
-        down(&pCifsFile->fh_sem);
-        if(pCifsFile->invalidHandle == FALSE) {
-                up(&pCifsFile->fh_sem);
-                FreeXid(xid);
-                return 0;
-        }
+	xid = GetXid();
+	down(&pCifsFile->fh_sem);
+	if(pCifsFile->invalidHandle == FALSE) {
+		up(&pCifsFile->fh_sem);
+		FreeXid(xid);
+		return 0;
+	}
 
 
-        cifs_sb = CIFS_SB(inode->i_sb);
-        pTcon = cifs_sb->tcon;
+	cifs_sb = CIFS_SB(inode->i_sb);
+	pTcon = cifs_sb->tcon;
 
-        full_path = build_path_from_dentry(file->f_dentry);
+	full_path = build_path_from_dentry(file->f_dentry);
 
-        cFYI(1, (" inode = 0x%p file flags are 0x%x for %s", inode, file->f_flags,full_path));
-        if ((file->f_flags & O_ACCMODE) == O_RDONLY)
-                desiredAccess = GENERIC_READ;
-        else if ((file->f_flags & O_ACCMODE) == O_WRONLY)
-                desiredAccess = GENERIC_WRITE;
-        else if ((file->f_flags & O_ACCMODE) == O_RDWR)
-                desiredAccess = GENERIC_READ | GENERIC_WRITE;
+	cFYI(1, (" inode = 0x%p file flags are 0x%x for %s", inode, file->f_flags,full_path));
+	if ((file->f_flags & O_ACCMODE) == O_RDONLY)
+		desiredAccess = GENERIC_READ;
+	else if ((file->f_flags & O_ACCMODE) == O_WRONLY)
+		desiredAccess = GENERIC_WRITE;
+	else if ((file->f_flags & O_ACCMODE) == O_RDWR) {
+		/* GENERIC_ALL is too much permission to request */
+		/* can cause unnecessary access denied on create */
+		/* desiredAccess = GENERIC_ALL; */
+		desiredAccess = GENERIC_READ | GENERIC_WRITE;
+	}
 
-        if (oplockEnabled)
-                oplock = REQ_OPLOCK;
-        else
-                oplock = FALSE;
+	if (oplockEnabled)
+		oplock = REQ_OPLOCK;
+	else
+		oplock = FALSE;
 
         /* BB pass O_SYNC flag through on file attributes .. BB */
 
         /* Also refresh inode by passing in file_info buf returned by SMBOpen
            and calling get_inode_info with returned buf (at least
            helps non-Unix server case */
-        buf = kmalloc(sizeof(FILE_ALL_INFO),GFP_KERNEL);
-        if(buf==0) {
-                up(&pCifsFile->fh_sem);
-                if (full_path)
-                        kfree(full_path);
-                FreeXid(xid);
-                return -ENOMEM;
-        }
-        rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
-                                CREATE_NOT_DIR, &netfid, &oplock, buf, cifs_sb->local_nls);
-        if (rc) {
-                up(&pCifsFile->fh_sem);
-                cFYI(1, ("cifs_open returned 0x%x ", rc));
-                cFYI(1, ("oplock: %d ", oplock));
-        } else {
-                pCifsFile->netfid = netfid;
-                pCifsFile->invalidHandle = FALSE;
-                up(&pCifsFile->fh_sem);
-                pCifsInode = CIFS_I(inode);
-                if(pCifsInode) {
-                        if (pTcon->ses->capabilities & CAP_UNIX)
-                                rc = cifs_get_inode_info_unix(&inode,
-                                                full_path, inode->i_sb);
-                        else
-                                rc = cifs_get_inode_info(&inode,
-                                                full_path, buf, inode->i_sb);
+	buf = kmalloc(sizeof(FILE_ALL_INFO),GFP_KERNEL);
+	if(buf==0) {
+		up(&pCifsFile->fh_sem);
+		if (full_path)
+			kfree(full_path);
+		FreeXid(xid);
+		return -ENOMEM;
+	}
+	rc = CIFSSMBOpen(xid, pTcon, full_path, disposition, desiredAccess,
+				CREATE_NOT_DIR, &netfid, &oplock, buf, cifs_sb->local_nls);
+	if (rc) {
+		up(&pCifsFile->fh_sem);
+		cFYI(1, ("cifs_open returned 0x%x ", rc));
+		cFYI(1, ("oplock: %d ", oplock));
+	} else {
+		pCifsFile->netfid = netfid;
+		pCifsFile->invalidHandle = FALSE;
+		up(&pCifsFile->fh_sem);
+		pCifsInode = CIFS_I(inode);
+		if(pCifsInode) {
+			if (pTcon->ses->capabilities & CAP_UNIX)
+				rc = cifs_get_inode_info_unix(&inode,
+						full_path, inode->i_sb);
+			else
+				rc = cifs_get_inode_info(&inode,
+						full_path, buf, inode->i_sb);
 
-                        if(oplock == OPLOCK_EXCLUSIVE) {
-                                pCifsInode->clientCanCacheAll =  TRUE;
-                                pCifsInode->clientCanCacheRead = TRUE;
-                                cFYI(1,("Exclusive Oplock granted on inode %p",file->f_dentry->d_inode));
-                        } else if(oplock == OPLOCK_READ) {
-                                pCifsInode->clientCanCacheRead = TRUE;
-                                pCifsInode->clientCanCacheAll =  FALSE;
-                        } else {
-                                pCifsInode->clientCanCacheRead = FALSE;
-                                pCifsInode->clientCanCacheAll =  FALSE;
-                        }
-                        cifs_relock_file(pCifsFile);
-                }
-        }
+			if(oplock == OPLOCK_EXCLUSIVE) {
+				pCifsInode->clientCanCacheAll =  TRUE;
+				pCifsInode->clientCanCacheRead = TRUE;
+				cFYI(1,("Exclusive Oplock granted on inode %p",file->f_dentry->d_inode));
+			} else if(oplock == OPLOCK_READ) {
+				pCifsInode->clientCanCacheRead = TRUE;
+				pCifsInode->clientCanCacheAll =  FALSE;
+			} else {
+				pCifsInode->clientCanCacheRead = FALSE;
+				pCifsInode->clientCanCacheAll =  FALSE;
+			}
+			cifs_relock_file(pCifsFile);
+		}
+	}
 
-        if (buf)
-                kfree(buf);
-        if (full_path)
-                kfree(full_path);
-        FreeXid(xid);
-        return rc;
+	if (buf)
+		kfree(buf);
+	if (full_path)
+		kfree(full_path);
+	FreeXid(xid);
+	return rc;
 }
 
 int
@@ -323,20 +348,20 @@ cifs_close(struct inode *inode, struct file *file)
 	cifs_sb = CIFS_SB(inode->i_sb);
 	pTcon = cifs_sb->tcon;
 	if (pSMBFile) {
-		pSMBFile->closePend	= TRUE;
-		spin_lock(&files_lock);
+		pSMBFile->closePend    = TRUE;
+		write_lock(&file->f_owner.lock);
 		if(pTcon) {
 			/* no sense reconnecting to close a file that is
 				already closed */
 			if (pTcon->tidStatus != CifsNeedReconnect) {
-				spin_unlock(&files_lock);
+				write_unlock(&file->f_owner.lock);
 				rc = CIFSSMBClose(xid,pTcon,pSMBFile->netfid);
-				spin_lock(&files_lock);
+				write_lock(&file->f_owner.lock);
 			}
 		}
 		list_del(&pSMBFile->flist);
 		list_del(&pSMBFile->tlist);
-		spin_unlock(&files_lock);
+		write_unlock(&file->f_owner.lock);
 		if(pSMBFile->search_resume_name)
 			kfree(pSMBFile->search_resume_name);
 		kfree(file->private_data);
@@ -350,7 +375,6 @@ cifs_close(struct inode *inode, struct file *file)
 		info on this inode, much less write behind and read ahead */
 		CIFS_I(inode)->clientCanCacheRead = FALSE;
 		CIFS_I(inode)->clientCanCacheAll  = FALSE;
-		invalidate_inode_pages(inode);
 	}
 	if((rc ==0) && CIFS_I(inode)->write_behind_rc)
 		rc = CIFS_I(inode)->write_behind_rc;
@@ -405,16 +429,15 @@ cifs_lock(struct file *file, int cmd, struct file_lock *pfLock)
 		cFYI(1, ("Posix "));
 	if (pfLock->fl_flags & FL_FLOCK)
 		cFYI(1, ("Flock "));
-/*	if (pfLock->fl_flags & FL_SLEEP) {
+	if (pfLock->fl_flags & FL_SLEEP) {
 		cFYI(1, ("Blocking lock "));
 		wait_flag = TRUE;
-	} */                          
-
+	}
 	if (pfLock->fl_flags & FL_ACCESS)
 		cFYI(1, ("Process suspended by mandatory locking - not implemented yet "));
 	if (pfLock->fl_flags & FL_LEASE)
 		cFYI(1, ("Lease on file - not implemented yet"));
-	if (pfLock->fl_flags & (~(FL_POSIX | FL_FLOCK | FL_ACCESS | FL_LEASE)))
+	if (pfLock->fl_flags & (~(FL_POSIX | FL_FLOCK | FL_SLEEP | FL_ACCESS | FL_LEASE)))
 		cFYI(1, ("Unknown lock flags 0x%x",pfLock->fl_flags));
 
 	if (pfLock->fl_type == F_WRLCK) {
@@ -561,7 +584,7 @@ cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
 {
 	struct address_space *mapping = page->mapping;
 	loff_t offset = (loff_t)page->index << PAGE_CACHE_SHIFT;
-	char * write_data = kmap(page);
+	char * write_data;
 	int rc = -EFAULT;
 	int bytes_written = 0;
 	struct cifs_sb_info *cifs_sb;
@@ -577,8 +600,7 @@ cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
 
 	/* figure out which file struct to use 
 	if (file->private_data == NULL) {
-	   kunmap(page);
-	   return -EBADF;
+		return -EBADF;
 	}     
 	 */
 	if (!mapping) {
@@ -588,20 +610,27 @@ cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
 	}
 
 	offset += (loff_t)from;
+	write_data = kmap(page);
 	write_data += from;
 
-	if((to > PAGE_CACHE_SIZE) || (from > to) || (offset > mapping->host->i_size)) {
+	if((to > PAGE_CACHE_SIZE) || (from > to)) {
 		kunmap(page);
 		return -EIO;
 	}
 
+	/* racing with truncate? */
+	if(offset > mapping->host->i_size) {
+		kunmap(page);
+		return 0; /* don't care */
+	}
+
 	/* check to make sure that we are not extending the file */
 	if(mapping->host->i_size - offset < (loff_t)to)
 		to = (unsigned)(mapping->host->i_size - offset); 
 		
 
 	cifsInode = CIFS_I(mapping->host);
-	read_lock(&GlobalSMBSeslock);
+	read_lock(&GlobalSMBSeslock); 
 	list_for_each_safe(tmp, tmp1, &cifsInode->openFileList) {            
 		open_file = list_entry(tmp,struct cifsFileInfo, flist);
 		/* We check if file is open for writing first */
@@ -620,8 +649,8 @@ cifs_partialpagewrite(struct page *page,unsigned from, unsigned to)
 				rc = bytes_written;
 			}
 			break;  /* now that we found a valid file handle
-                                and tried to write to it we are done, no
-                                sense continuing to loop looking for another */
+				and tried to write to it we are done, no
+				sense continuing to loop looking for another */
 		}
 		if(tmp->next == NULL) {
 			cFYI(1,("File instance %p removed",tmp));
@@ -653,7 +682,7 @@ cifs_writepages(struct address_space *mapping, struct writeback_control *wbc)
 #endif
 
 static int
-cifs_writepage(struct page* page)
+cifs_writepage(struct page* page, struct writeback_control *wbc)
 {
 	int rc = -EFAULT;
 	int xid;
@@ -664,11 +693,7 @@ cifs_writepage(struct page* page)
 	
 	rc = cifs_partialpagewrite(page,0,PAGE_CACHE_SIZE);
 	SetPageUptodate(page); /* BB add check for error and Clearuptodate? */
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,20))
 	unlock_page(page);
-#else
-	UnlockPage(page);
-#endif
 	page_cache_release(page);	
 	FreeXid(xid);
 	return rc;
@@ -731,16 +756,10 @@ cifs_fsync(struct file *file, struct dentry *dentry, int datasync)
 
 	cFYI(1, ("Sync file - name: %s datasync: 0x%x ", 
 		dentry->d_name.name, datasync));
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))	
-	rc = filemap_fdatasync(inode->i_mapping);
-#else
-	filemap_fdatasync(inode->i_mapping);
-#endif
-
-
+	
+	rc = filemap_fdatawrite(inode->i_mapping);
 	if(rc == 0)
 		CIFS_I(inode)->write_behind_rc = 0;
-
 	FreeXid(xid);
 	return rc;
 }
@@ -790,15 +809,12 @@ int cifs_flush(struct file *file)
 	/* deal with writebehind errors */
 	/* unlock inode for writing */
 	/* filemapfdatawrite appears easier for the time being */
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,17))
-	rc = filemap_fdatasync(inode->i_mapping);
-#else
-	filemap_fdatasync(inode->i_mapping);
-#endif
 
-	cFYI(1,("Flush inode %p file %p rc %d",inode,file,rc));
+	rc = filemap_fdatawrite(inode->i_mapping);
 	if(rc == 0) /* reset wb rc if we were able to write out dirty pages */
 		CIFS_I(inode)->write_behind_rc = 0;
+		
+	cFYI(1,("Flush inode %p file %p rc %d",inode,file,rc));
 
 	return rc;
 }
@@ -829,24 +845,20 @@ cifs_read(struct file * file, char *read_data, size_t read_size,
 	open_file = (struct cifsFileInfo *)file->private_data;
 
 	for (total_read = 0,current_offset=read_data; read_size > total_read;
-		total_read += bytes_read,current_offset+=bytes_read) {
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
+				total_read += bytes_read,current_offset+=bytes_read) {
 		current_read_size = min_t(const int,read_size - total_read,cifs_sb->rsize);
-#else
-                current_read_size = min(read_size - total_read,cifs_sb->rsize);
-#endif
 		rc = -EAGAIN;
 		while(rc == -EAGAIN) {
 			if ((open_file->invalidHandle) && (!open_file->closePend)) {
 				rc = cifs_reopen_file(file->f_dentry->d_inode,file);
 				if(rc != 0)
-				break;
+					break;
 			}
 
 			rc = CIFSSMBRead(xid, pTcon,
-				open_file->netfid,
-				current_read_size, *poffset,
-				&bytes_read, &current_offset);
+				 open_file->netfid,
+				 current_read_size, *poffset,
+				 &bytes_read, &current_offset);
 		}
 		if (rc || (bytes_read == 0)) {
 			if (total_read) {
@@ -881,7 +893,6 @@ int cifs_file_mmap(struct file * file, struct vm_area_struct * vma)
 	return rc;
 }
 
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,5,0))
 static void cifs_copy_cache_pages(struct address_space *mapping, 
 		struct list_head *pages, int bytes_read, 
 		char *data,struct pagevec * plru_pvec)
@@ -920,18 +931,15 @@ static void cifs_copy_cache_pages(struct address_space *mapping,
 			__pagevec_lru_add(plru_pvec);
 		flush_dcache_page(page);
 		SetPageUptodate(page);
-		kunmap_atomic(page,KM_USER0);
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,20))
+		kunmap_atomic(target,KM_USER0);
 		unlock_page(page);
-#else
-		UnlockPage(page);
-#endif
 		page_cache_release(page);
 		data += PAGE_CACHE_SIZE;
 	}
 	return;
 }
 
+
 static int
 cifs_readpages(struct file *file, struct address_space *mapping,
 		struct list_head *page_list, unsigned num_pages)
@@ -977,11 +985,8 @@ cifs_readpages(struct file *file, struct address_space *mapping,
 		
 		read_size = (num_pages - i) * PAGE_CACHE_SIZE;
 		/* Read size needs to be in multiples of one page */
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 		read_size = min_t(const unsigned int,read_size,cifs_sb->rsize & PAGE_CACHE_MASK);
-#else
-                read_size = min(read_size,cifs_sb->rsize & PAGE_CACHE_MASK);
-#endif
+
 		rc = -EAGAIN;
 		while(rc == -EAGAIN) {
 			if ((open_file->invalidHandle) && (!open_file->closePend)) {
@@ -995,11 +1000,9 @@ cifs_readpages(struct file *file, struct address_space *mapping,
 				read_size, offset,
 				&bytes_read, &smb_read_data);
 		}
-
 		if ((rc < 0) || (smb_read_data == NULL)) {
 			cFYI(1,("Read error in readpages: %d",rc));
-			/* clean up remaing pages off list */
-            
+			/* clean up remaing pages off list */            
 			spin_lock(&mapping->page_lock);
 			while (!list_empty(page_list) && (i < num_pages)) {
 				page = list_entry(page_list->prev, struct page, list);
@@ -1039,7 +1042,6 @@ cifs_readpages(struct file *file, struct address_space *mapping,
 	FreeXid(xid);
 	return rc;
 }
-#endif
 
 static int
 cifs_readpage(struct file *file, struct page *page)
@@ -1081,11 +1083,7 @@ cifs_readpage(struct file *file, struct page *page)
 
 io_error:
 	kunmap(page);
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,20))
 	unlock_page(page);
-#else
-	UnlockPage(page);
-#endif
 
 	page_cache_release(page);
 	FreeXid(xid);
@@ -1145,13 +1143,8 @@ fill_in_inode(struct inode *tmp_inode,
 
 	/* can not fill in nlink here as in qpathinfo version and Unx search */
 	tmp_inode->i_size = pfindData->EndOfFile;
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15)) 
 	tmp_inode->i_blocks =
 		(tmp_inode->i_blksize - 1 + pfindData->AllocationSize) >> tmp_inode->i_blkbits;
-#else
-	tmp_inode->i_blocks =
-		(tmp_inode->i_blksize - 1 + pfindData->AllocationSize) >> 14;
-#endif
 	if (pfindData->AllocationSize < pfindData->EndOfFile)
 		cFYI(1, ("Possible sparse file: allocation size less than end of file "));
 	cFYI(1,
@@ -1173,7 +1166,7 @@ fill_in_inode(struct inode *tmp_inode,
 	} else {
 		cFYI(1, (" Init special inode "));
 		init_special_inode(tmp_inode, tmp_inode->i_mode,
-				   kdev_t_to_nr(tmp_inode->i_rdev));
+				   tmp_inode->i_rdev);
 	}
 }
 
@@ -1224,13 +1217,8 @@ unix_fill_in_inode(struct inode *tmp_inode,
 	pfindData->NumOfBytes = le64_to_cpu(pfindData->NumOfBytes);
 	pfindData->EndOfFile = le64_to_cpu(pfindData->EndOfFile);
 	tmp_inode->i_size = pfindData->EndOfFile;
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 	tmp_inode->i_blocks =
                 (tmp_inode->i_blksize - 1 + pfindData->NumOfBytes) >> tmp_inode->i_blkbits;
-#else
-	tmp_inode->i_blocks =
-                (tmp_inode->i_blksize - 1 + pfindData->NumOfBytes) >> 14;
-#endif
 
 	if (S_ISREG(tmp_inode->i_mode)) {
 		cFYI(1, (" File inode "));
@@ -1248,7 +1236,7 @@ unix_fill_in_inode(struct inode *tmp_inode,
 	} else {
 		cFYI(1, (" Init special inode "));
 		init_special_inode(tmp_inode, tmp_inode->i_mode,
-				   kdev_t_to_nr(tmp_inode->i_rdev));
+				   tmp_inode->i_rdev);
 	}
 }
 
@@ -1259,7 +1247,6 @@ construct_dentry(struct qstr *qstring, struct file *file,
 	struct dentry *tmp_dentry;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsTconInfo *pTcon;
-	struct cifsInodeInfo *pCifsI;
 
 	cFYI(1, ("For %s ", qstring->name));
 	cifs_sb = CIFS_SB(file->f_dentry->d_sb);
@@ -1274,23 +1261,6 @@ construct_dentry(struct qstr *qstring, struct file *file,
 		if(*ptmp_inode == NULL) {
 	                *ptmp_inode = new_inode(file->f_dentry->d_sb);
 			d_instantiate(tmp_dentry, *ptmp_inode);
-                        pCifsI = CIFS_I(*ptmp_inode);
-                        INIT_LIST_HEAD(&pCifsI->openFileList);
-                        /* can not enable caching for this inode
-                        until a file instance is open and we
-                        can check the oplock flag on the open
-                        response */
-                        (*ptmp_inode)->i_blksize = CIFS_MAX_MSGSIZE;
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
-                        (*ptmp_inode)->i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
-#endif
-                        pCifsI->clientCanCacheRead = FALSE;
-                        pCifsI->clientCanCacheAll = FALSE;
-                        pCifsI->time = 0;
-                        /* do not need to set cifs Attrs since
-                                they are about to be overwritten
-                                in fill_in_inode */
-                        atomic_set(&pCifsI->inUse, 0);
 		}
 	} else {
 		tmp_dentry = d_alloc(file->f_dentry, qstring);
@@ -1300,25 +1270,6 @@ construct_dentry(struct qstr *qstring, struct file *file,
 		}
 			
 		*ptmp_inode = new_inode(file->f_dentry->d_sb);
-		if(ptmp_inode) {
-			pCifsI = CIFS_I(*ptmp_inode);
-			INIT_LIST_HEAD(&pCifsI->openFileList);
-			/* can not enable caching for this inode 
-			until a file instance is open and we
-			can check the oplock flag on the open
-			response */
-	                (*ptmp_inode)->i_blksize = CIFS_MAX_MSGSIZE;
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
-			(*ptmp_inode)->i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
-#endif
-			pCifsI->clientCanCacheRead = FALSE;
-			pCifsI->clientCanCacheAll = FALSE;
-			pCifsI->time = 0;
-			/* do not need to set cifs Attrs since
-				they are about to be overwritten 
-				in fill_in_inode */
-			atomic_set(&pCifsI->inUse, 0);
-		}
 		tmp_dentry->d_op = &cifs_dentry_ops;
 		cFYI(0, (" instantiate dentry 0x%p with inode 0x%p ",
 			 tmp_dentry, *ptmp_inode));
@@ -1327,13 +1278,12 @@ construct_dentry(struct qstr *qstring, struct file *file,
 	}
 
 	tmp_dentry->d_time = jiffies;
-	cFYI(1, ("i_blksize = %ld", (*ptmp_inode)->i_blksize));
 	*pnew_dentry = tmp_dentry;
 }
 
 static void reset_resume_key(struct file * dir_file, 
-						unsigned char * filename, 
-						unsigned int len) {
+				unsigned char * filename, 
+				unsigned int len,int Unicode,struct nls_table * nls_tab) {
 	struct cifsFileInfo *cifsFile;
 
 	cifsFile = (struct cifsFileInfo *)dir_file->private_data;
@@ -1343,12 +1293,20 @@ static void reset_resume_key(struct file * dir_file,
 		kfree(cifsFile->search_resume_name);
 	}
 
+	if(Unicode) 
+		len *= 2;
 	cifsFile->resume_name_length = len;
+
 	cifsFile->search_resume_name = 
-			kmalloc(cifsFile->resume_name_length, GFP_KERNEL);
-	memcpy(cifsFile->search_resume_name, filename, 
+		kmalloc(cifsFile->resume_name_length, GFP_KERNEL);
+
+	if(Unicode)
+		cifs_strtoUCS((wchar_t *) cifsFile->search_resume_name,
+			filename, len, nls_tab);
+	else
+		memcpy(cifsFile->search_resume_name, filename, 
 		   cifsFile->resume_name_length);
-	cFYI(1,("Reset resume key to: %s",filename));
+	cFYI(1,("Reset resume key to: %s with len %d",filename,len));
 	return;
 }
 
@@ -1373,7 +1331,6 @@ cifs_filldir(struct qstr *pqstring, FILE_DIRECTORY_INFO * pfindData,
 	if(rc) {
 		/* due to readdir error we need to recalculate resume 
 		key so next readdir will restart on right entry */
-		reset_resume_key(file, pfindData->FileName, pqstring->len);
 		cFYI(1,("Error %d on filldir of %s",rc ,pfindData->FileName));
 	}
 	dput(tmp_dentry);
@@ -1400,7 +1357,6 @@ cifs_filldir_unix(struct qstr *pqstring,
 	if(rc) {
 		/* due to readdir error we need to recalculate resume 
 			key so next readdir will restart on right entry */
-		reset_resume_key(file, pUnixFindData->FileName,pqstring->len);
 		cFYI(1,("Error %d on filldir of %s",rc ,pUnixFindData->FileName));
 	}
 	dput(tmp_dentry);
@@ -1591,6 +1547,8 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 							/* do not end search if
 								kernel not ready to take
 								remaining entries yet */
+							reset_resume_key(file, pfindData->FileName,qstring.len,
+								Unicode, cifs_sb->local_nls);
 							findParms.EndofSearch = 0;
 							break;
 						}
@@ -1629,6 +1587,8 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 								kernel not ready to take
 								remaining entries yet */
 							findParms.EndofSearch = 0;
+							reset_resume_key(file, pfindDataUnix->FileName,
+								qstring.len,Unicode,cifs_sb->local_nls);
 							break;
 						}
 						file->f_pos++;
@@ -1667,7 +1627,6 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 				break;
 			}
 			searchHandle = cifsFile->netfid;
-			cFYI(1,("FindNext resume name: %s len: %d",cifsFile->search_resume_name,cifsFile->resume_name_length));
 			rc = CIFSFindNext(xid, pTcon, pfindData,
 				&findNextParms, searchHandle, 
 				cifsFile->search_resume_name,
@@ -1779,6 +1738,8 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 								kernel not ready to take
 								remaining entries yet */
 								findNextParms.EndofSearch = 0;
+								reset_resume_key(file, pfindData->FileName,qstring.len,
+									Unicode,cifs_sb->local_nls);
 								break;
 							}
 							file->f_pos++;
@@ -1819,6 +1780,8 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 								kernel not ready to take
 								remaining entries yet */
 								findNextParms.EndofSearch = 0;
+								reset_resume_key(file, pfindDataUnix->FileName,qstring.len,
+									Unicode,cifs_sb->local_nls);
 								break;
 							}
 							file->f_pos++;
@@ -1845,27 +1808,11 @@ cifs_readdir(struct file *file, void *direntry, filldir_t filldir)
 	return rc;
 }
 
-static int cifs_prepare_write(struct file *file, struct page *page,
-                        unsigned from, unsigned to)
-{
-        if (!Page_Uptodate(page)) {
-                if (to - from != PAGE_CACHE_SIZE) {
-                        void *kaddr = kmap_atomic(page, KM_USER0);
-                        memset(kaddr, 0, from);
-                        memset(kaddr + to, 0, PAGE_CACHE_SIZE - to);
-                        flush_dcache_page(page);
-                        kunmap_atomic(kaddr, KM_USER0);
-                }
-                SetPageUptodate(page);
-        }
-        return 0;
-}
-
 struct address_space_operations cifs_addr_ops = {
 	.readpage = cifs_readpage,
-/*	.readpages = cifs_readpages, */
+	.readpages = cifs_readpages,
 	.writepage = cifs_writepage,
-	.prepare_write = cifs_prepare_write,
+	.prepare_write = simple_prepare_write,
 	.commit_write = cifs_commit_write,
 	.sync_page = cifs_sync_page,
 	/*.direct_IO = */
@@ -1873,21 +1820,21 @@ struct address_space_operations cifs_addr_ops = {
 
 struct address_space_operations cifs_addr_ops_writethrough = {
 	.readpage = cifs_readpage,
-/*	.readpages = cifs_readpages,*/
+	.readpages = cifs_readpages,
 	.writepage = cifs_writepage,
-	.prepare_write = cifs_prepare_write,
+	.prepare_write = simple_prepare_write,
 	.commit_write = cifs_commit_write,
 	.sync_page = cifs_sync_page,
 	/*.direct_IO = 	 */
-	/*.direct_IO = */
 };
 
 struct address_space_operations cifs_addr_ops_nocache = {
 	.readpage = cifs_readpage,
-/*	.readpages = cifs_readpages,*/
+	.readpages = cifs_readpages,
 	.writepage = cifs_writepage,
-	.prepare_write = cifs_prepare_write,
+	.prepare_write = simple_prepare_write,
 	.commit_write = cifs_commit_write,
 	.sync_page = cifs_sync_page,
+	/*.direct_IO = */
 };
 
