@@ -19,9 +19,9 @@
  *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
  */
 #include <linux/fs.h>
+#include <linux/buffer_head.h>
 #include <linux/stat.h>
 #include <linux/pagemap.h>
-#include <linux/version.h>
 #include <asm/div64.h>
 #include "cifsfs.h"
 #include "cifspdu.h"
@@ -30,28 +30,6 @@
 #include "cifs_debug.h"
 #include "cifs_fs_sb.h"
 
-struct inode * get_cifs_inode(struct super_block * sb)
-{
-	struct inode * newinode;
-	newinode = new_inode(sb);
-	cFYI(1,("got new inode %p",newinode));
-	if(newinode) {    
-		struct cifsInodeInfo * cifsInfo = CIFS_I(newinode);
-		cifsInfo->clientCanCacheRead = FALSE;
-		cifsInfo->clientCanCacheAll = FALSE;
-		INIT_LIST_HEAD(&cifsInfo->openFileList);
-		cifsInfo->cifsAttrs = 0x20;   /* default */
-		newinode->i_blksize = CIFS_MAX_MSGSIZE;
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
-		newinode->i_blkbits = 14;  /* 2**14 = CIFS_MAX_MSGSIZE */
-#endif
-		atomic_set(&cifsInfo->inUse, 0);
-		cifsInfo->time = 0;
-	}
-	return newinode;
-
-}
-
 int
 cifs_get_inode_info_unix(struct inode **pinode,
 			 const unsigned char *search_path,
@@ -107,7 +85,7 @@ cifs_get_inode_info_unix(struct inode **pinode,
 
 		/* get new inode */
 		if (*pinode == NULL) {
-			*pinode = get_cifs_inode(sb);
+			*pinode = new_inode(sb);
 		}
 		inode = *pinode;
 
@@ -152,13 +130,8 @@ cifs_get_inode_info_unix(struct inode **pinode,
 	and blkbits set in superblock so 2**blkbits and blksize will match */
 /*		inode->i_blksize =
 		    (pTcon->ses->server->maxBuf - MAX_CIFS_HDR_SIZE) & 0xFFFFFE00;*/
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 		inode->i_blocks = 
 	                (inode->i_blksize - 1 + findData.NumOfBytes) >> inode->i_blkbits;
-#else
-		inode->i_blocks = 
-	                (inode->i_blksize - 1 + findData.NumOfBytes) >> 14;
-#endif
 
 		if (findData.NumOfBytes < findData.EndOfFile)
 			cFYI(1, ("Server inconsistency Error: it says allocation size less than end of file "));
@@ -181,7 +154,7 @@ cifs_get_inode_info_unix(struct inode **pinode,
 		} else {
 			cFYI(1, (" Init special inode "));
 			init_special_inode(inode, inode->i_mode,
-					   kdev_t_to_nr(inode->i_rdev));
+					   inode->i_rdev);
 		}
 	}
 	FreeXid(xid);
@@ -255,7 +228,7 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 
 		/* get new inode */
 		if (*pinode == NULL) {
-			*pinode = get_cifs_inode(sb);
+			*pinode = new_inode(sb);
 		}
 
 		inode = *pinode;
@@ -301,17 +274,9 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 		}
 		inode->i_size = le64_to_cpu(pfindData->EndOfFile);
 		pfindData->AllocationSize = le64_to_cpu(pfindData->AllocationSize);
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,15))
 		inode->i_blocks =
 	                (inode->i_blksize - 1 + pfindData->AllocationSize) >> inode->i_blkbits;
-#else
-		inode->i_blocks =
-	                (inode->i_blksize - 1 + pfindData->AllocationSize) >> 14;
-#endif
 
-		cFYI(1,
-		     (" Size %ld and blocks %ld ",
-		      (unsigned long) inode->i_size, inode->i_blocks));
 		inode->i_nlink = le32_to_cpu(pfindData->NumberOfLinks);
 
 		/* BB fill in uid and gid here? with help from winbind? 
@@ -333,7 +298,7 @@ cifs_get_inode_info(struct inode **pinode, const unsigned char *search_path,
 			inode->i_op = &cifs_symlink_inode_ops;
 		} else {
 			init_special_inode(inode, inode->i_mode,
-					   kdev_t_to_nr(inode->i_rdev));
+					   inode->i_rdev);
 		}
 	}
 	if(buf)
@@ -380,6 +345,8 @@ cifs_unlink(struct inode *inode, struct dentry *direntry)
 
 	if (!rc) {
 		direntry->d_inode->i_nlink--;
+	} else if (rc == -ENOENT) {
+		d_drop(direntry);
 	} else if (rc == -ETXTBSY) {
 		int oplock = FALSE;
 		__u16 netfid;
@@ -593,6 +560,20 @@ cifs_revalidate(struct dentry *direntry)
 	char *full_path;
 	struct cifs_sb_info *cifs_sb;
 	struct cifsInodeInfo *cifsInode;
+	loff_t local_size;
+        struct timespec local_mtime;	
+
+	if(direntry->d_inode == NULL)
+		return -ENOENT;
+
+	cifsInode = CIFS_I(direntry->d_inode);
+
+	if(cifsInode == NULL)
+		return -ENOENT;
+
+	/* no sense revalidating inode info on file that no one can write */
+	if(CIFS_I(direntry->d_inode)->clientCanCacheRead)
+		return rc;
 
 	xid = GetXid();
 
@@ -605,13 +586,9 @@ cifs_revalidate(struct dentry *direntry)
 	      direntry->d_inode->i_count.counter, direntry,
 	      direntry->d_time, jiffies));
 
-	cifsInode = CIFS_I(direntry->d_inode);
-	/* BB add check - do not need to revalidate oplocked files */
-
-	if (time_before(jiffies, cifsInode->time + HZ)) {
+	if (time_before(jiffies, cifsInode->time + HZ) && lookupCacheEnabled) {
 	    if((S_ISREG(direntry->d_inode->i_mode) == 0) || 
-			(direntry->d_inode->i_nlink == 1) || 
-			(lookupCacheEnabled == 0)) {
+			(direntry->d_inode->i_nlink == 1)) {  
 			if (full_path)
 				kfree(full_path);
 			FreeXid(xid);
@@ -620,7 +597,18 @@ cifs_revalidate(struct dentry *direntry)
 			cFYI(1,("Have to revalidate file due to hardlinks"));
 		}            
 	}
+	
+	/* save mtime and size */
+	local_mtime = direntry->d_inode->i_mtime;
+	local_size  = direntry->d_inode->i_size;
 
+	/* need to write out dirty pages here if any before getting possibly 
+		stale time from server */
+	if(direntry->d_inode->i_mapping) {
+		/* do we need to lock inode until after invalidate completes below? */
+		filemap_fdatawrite(direntry->d_inode->i_mapping);
+		filemap_fdatawrite(direntry->d_inode->i_mapping);
+	}
 	if (cifs_sb->tcon->ses->capabilities & CAP_UNIX) {
 		rc = cifs_get_inode_info_unix(&direntry->d_inode, full_path,
 					 direntry->d_sb);
@@ -640,8 +628,17 @@ cifs_revalidate(struct dentry *direntry)
 	}
 	/* should we remap certain errors, access denied?, to zero */
 
-	/* BB if not oplocked, invalidate inode pages if mtime has changed */
+	/* if not oplocked, we invalidate inode pages if mtime 
+	   or file size has changed on server */
 
+	if(timespec_equal(&local_mtime,&direntry->d_inode->i_mtime) && 
+		(local_size == direntry->d_inode->i_size)) {
+		cFYI(1,("inode unchanged on server"));
+	} else {
+		/* file has changed on server */
+		invalidate_remote_inode(direntry->d_inode);
+	}
+	
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
@@ -649,13 +646,13 @@ cifs_revalidate(struct dentry *direntry)
 	return rc;
 }
 
-/* int cifs_getattr(struct vfsmount *mnt, struct dentry *dentry, struct kstat *stat)
+int cifs_getattr(struct vfsmount *mnt, struct dentry *dentry, struct kstat *stat)
 {
 	int err = cifs_revalidate(dentry);
 	if (!err)
 		generic_fillattr(dentry->d_inode, stat);
 	return err;
-} */
+}
 
 void
 cifs_truncate_file(struct inode *inode)
@@ -735,32 +732,6 @@ cifs_truncate_file(struct inode *inode)
 	return;
 }
 
-static int cifs_trunc_page(struct address_space *mapping, loff_t from)
-{
-        unsigned long index = from >> PAGE_CACHE_SHIFT;
-        unsigned offset = from & (PAGE_CACHE_SIZE-1);
-        struct page *page;
-        char *kaddr;
-        int rc = 0;
-
-        page = grab_cache_page(mapping, index);
-        if (!page)
-                return -ENOMEM;
-
-        kaddr = kmap_atomic(page, KM_USER0);
-        memset(kaddr + offset, 0, PAGE_CACHE_SIZE - offset);
-        flush_dcache_page(page);
-        kunmap_atomic(kaddr, KM_USER0);
-        set_page_dirty(page);
-#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,4,20))
-        unlock_page(page);
-#else
-	UnlockPage(page);
-#endif
-        page_cache_release(page);
-        return rc;
-}
-
 int
 cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 {
@@ -795,29 +766,29 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 	cFYI(1, (" Changing attributes 0x%x", attrs->ia_valid));
 
 	if (attrs->ia_valid & ATTR_SIZE) {
-		read_lock(&GlobalSMBSeslock);
+		read_lock(&GlobalSMBSeslock); 
 		/* To avoid spurious oplock breaks from server, in the case
 			of inodes that we already have open, avoid doing path
 			based setting of file size if we can do it by handle.
 			This keeps our caching token (oplock) and avoids
 			timeouts when the local oplock break takes longer to flush
-			writebehind data than the SMB timeout for the SetPathInfo
+			writebehind data than the SMB timeout for the SetPathInfo 
 			request would allow */
-		list_for_each(tmp, &cifsInode->openFileList) {
+		list_for_each(tmp, &cifsInode->openFileList) {            
 			open_file = list_entry(tmp,struct cifsFileInfo, flist);
 			/* We check if file is open for writing first */
 			if((open_file->pfile) &&
-			   ((open_file->pfile->f_flags & O_RDWR) ||
-			    (open_file->pfile->f_flags & O_WRONLY))) {
+				((open_file->pfile->f_flags & O_RDWR) || 
+				 (open_file->pfile->f_flags & O_WRONLY))) {
 				if(open_file->invalidHandle == FALSE) {
-					/* we found a valid, writeable network file
+					/* we found a valid, writeable network file 
 					handle to use to try to set the file size */
 					__u16 nfid = open_file->netfid;
 					__u32 npid = open_file->pid;
 					read_unlock(&GlobalSMBSeslock);
 					found = TRUE;
 					rc = CIFSSMBSetFileSize(xid, pTcon, attrs->ia_size,
-						nfid,npid,FALSE);
+					   nfid,npid,FALSE);
 					cFYI(1,("SetFileSize by handle (setattrs) rc = %d",rc));
 				/* Do not need reopen and retry on EAGAIN since we will
 					retry by pathname below */
@@ -835,7 +806,7 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 			because no valid, writeable file handle for it was found or
 			because there was an error setting it by handle */
 			rc = CIFSSMBSetEOF(xid, pTcon, full_path, attrs->ia_size,FALSE,
-				cifs_sb->local_nls);
+				   cifs_sb->local_nls);
 			cFYI(1,(" SetEOF by path (setattrs) rc = %d",rc));
 		}
 
@@ -845,7 +816,6 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 
 		if (rc == 0) {
 			rc = vmtruncate(direntry->d_inode, attrs->ia_size);
-			cifs_trunc_page(direntry->d_inode->i_mapping, direntry->d_inode->i_size);
 		}
 	}
 	if (attrs->ia_valid & ATTR_UID) {
@@ -915,11 +885,9 @@ cifs_setattr(struct dentry *direntry, struct iattr *attrs)
 		rc = CIFSSMBSetTimes(xid, pTcon, full_path, &time_buf,
 				cifs_sb->local_nls);
 	}
-	
+
 	/* do not  need local check to inode_check_ok since the server does that */
 	inode_setattr(direntry->d_inode, attrs);
-
-
 	if (full_path)
 		kfree(full_path);
 	FreeXid(xid);
