@@ -23,7 +23,6 @@
 #include <linux/list.h>
 #include <linux/wait.h>
 #include <linux/net.h>
-#include <linux/version.h>
 #include <asm/uaccess.h>
 #include <asm/processor.h>
 #include "cifspdu.h"
@@ -79,7 +78,7 @@ DeleteMidQEntry(struct mid_q_entry *midEntry)
 	list_del(&midEntry->qhead);
 	atomic_dec(&midCount);
 	spin_unlock(&GlobalMid_Lock);
-	buf_release(midEntry->resp_buf);
+	cifs_buf_release(midEntry->resp_buf);
 	kmem_cache_free(cifs_mid_cachep, midEntry);
 }
 
@@ -87,7 +86,7 @@ struct oplock_q_entry *
 AllocOplockQEntry(struct inode * pinode, __u16 fid, struct cifsTconInfo * tcon)
 {
 	struct oplock_q_entry *temp;
-	if ((pinode == NULL) || (tcon == NULL)) {
+	if ((pinode== NULL) || (tcon == NULL)) {
 		cERROR(1, ("Null parms passed to AllocOplockQEntry"));
 		return NULL;
 	}
@@ -97,8 +96,8 @@ AllocOplockQEntry(struct inode * pinode, __u16 fid, struct cifsTconInfo * tcon)
 		return temp;
 	else {
 		temp->pinode = pinode;
-		temp->netfid = fid;
 		temp->tcon = tcon;
+		temp->netfid = fid;
 		spin_lock(&GlobalMid_Lock);
 		list_add_tail(&temp->qhead, &GlobalOplock_Q);
 		spin_unlock(&GlobalMid_Lock);
@@ -173,7 +172,7 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 {
 	int rc = 0;
 	unsigned int receive_len;
-	long timeout = 10 * HZ;
+	long timeout;
 	struct mid_q_entry *midQ;
 
 	if ((ses == NULL) || (ses->server == NULL)) {
@@ -219,7 +218,7 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 
 	midQ->midState = MID_REQUEST_SUBMITTED;
 	rc = smb_send(ses->server->ssocket, in_buf, in_buf->smb_buf_length,
-		      (struct sockaddr *) &(ses->server->sockAddr));
+		      (struct sockaddr *) &(ses->server->addr.sockAddr));
 	up(&ses->server->tcpSem);
 	if (long_op == -1)
 		goto cifs_no_response_exit;
@@ -235,8 +234,10 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 	/* wait for 15 seconds or until woken up due to response arriving or 
 	   due to last connection to this server being unmounted */
 
-	timeout = interruptible_sleep_on_timeout(&ses->server->response_q,timeout);
-
+	timeout = wait_event_interruptible_timeout(ses->server->response_q,
+				midQ->
+				midState & MID_RESPONSE_RECEIVED,
+				timeout);
 	if (signal_pending(current)) {
 		cFYI(1, ("CIFS: caught signal"));
 		DeleteMidQEntry(midQ);
@@ -250,8 +251,12 @@ SendReceive(const unsigned int xid, struct cifsSesInfo *ses,
 		} else {
 			cFYI(1,("No response buffer"));
 			if(midQ->midState == MID_REQUEST_SUBMITTED) {
+				if(ses->server->tcpStatus == CifsExiting)
+					rc = -EHOSTDOWN;
+				else {
 				ses->server->tcpStatus = CifsNeedReconnect;
 				midQ->midState = MID_RETRY_NEEDED;
+				}
 			}
 
 			if(midQ->midState == MID_RETRY_NEEDED) {
